/*
    Pirate Bulletin Board System
    Copyright (C) 1990, Edward Luke, lush@Athena.EE.MsState.EDU
    Eagles Bulletin Board System
    Copyright (C) 1992, Raymond Rocker, rocker@rock.b11.ingr.com
                        Guy Vega, gtvega@seabass.st.usm.edu
                        Dominic Tynes, dbtynes@seabass.st.usm.edu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "bbs.h"
#ifdef AIX
# include <sys/select.h>
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>

#define OBUFSIZE  (4096)
#define IBUFSIZE  (256)

#define INPUT_ACTIVE 0
#define INPUT_IDLE 1

extern int dumb_term ;

char outbuf[OBUFSIZE] ;
int obufsize = 0 ;

char inbuf[IBUFSIZE] ;
int ibufsize = 0 ;
int icurrchar = 0 ;

static int i_mode = INPUT_ACTIVE;

void
hit_alarm_clock()
{
#ifdef PERMS
    if (HAS_PERM(PERM_NOTIMEOUT))
#else
    if (currentuser.userlevel == 255)
#endif
        return;
    if(i_mode == INPUT_IDLE) {
        clear();
        fprintf(stderr,"Idle timeout exceeded! Booting...\n") ;
        kill(getpid(),SIGHUP) ;
    }
    i_mode = INPUT_IDLE ;
    alarm(IDLE_TIMEOUT) ;
}

init_alarm()
{
    signal(SIGALRM,hit_alarm_clock) ;
    alarm(IDLE_TIMEOUT) ;
}

oflush()
{
    if(obufsize)
      write(1,outbuf,obufsize) ;
    obufsize = 0 ;
}

output(s,len)
char *s ;
{
    /* Invalid if len >= OBUFSIZE */

    if(obufsize+len > OBUFSIZE) {  /* doin a oflush */
        write(1,outbuf,obufsize) ;
        obufsize = 0 ;
    }
    bcopy(s,outbuf+obufsize,len) ;
    obufsize+=len ;
}

ochar(c)
{
    if(obufsize > OBUFSIZE-1) {  /* doin a oflush */
        write(1,outbuf,obufsize) ;
        obufsize = 0 ;
    }
    outbuf[obufsize++] = c ;
}

int i_newfd  = 0 ;
struct timeval i_to, *i_top = NULL ;
int (*flushf)() = NULL ;

add_io(fd,timeout)
int fd ;
int timeout ;
{
    i_newfd = fd ;
    if(timeout) {
        i_to.tv_sec = timeout ;
        i_to.tv_usec = 0 ;
        i_top = &i_to ;
    } else i_top = NULL ;
}

add_flush(flushfunc)
int (*flushfunc)() ;
{
    flushf = flushfunc ;
}

num_in_buf()
{
    return icurrchar - ibufsize ;
}

int
igetch()
{
  igetagain:
    if(ibufsize == icurrchar) {
        fd_set readfds ;
        struct timeval to ;
        int sr ;

        to.tv_sec = 0 ;
        to.tv_usec = 0 ;
        FD_ZERO(&readfds) ;
        FD_SET(0,&readfds) ;
        if(i_newfd)
          FD_SET(i_newfd,&readfds) ;
        if((sr = select(getdtablesize(),&readfds, NULL, NULL, &to)) <= 0) {
            if(flushf)
              (*flushf)() ;
            if(dumb_term)
              oflush() ;
            else
              refresh() ;
            FD_ZERO(&readfds) ;
            FD_SET(0,&readfds) ;
            if(i_newfd)
              FD_SET(i_newfd,&readfds) ;
            while((sr = select(getdtablesize(),&readfds, 
                               NULL, NULL, i_top)) <0) {
                if(errno == EINTR)
                  continue ;
                else {
                    perror("select") ;
                    fprintf(stderr,"abnormal select conditions\n") ;
                    return -1 ;
                }
            }
            if(sr == 0)
              return I_TIMEOUT ;
        }
        if(i_newfd && FD_ISSET(i_newfd,&readfds))
          return I_OTHERDATA ;
        while((ibufsize = read(0,inbuf,IBUFSIZE)) <= 0) {
            if(ibufsize == 0)
              longjmp(byebye,-1) ;
            if(ibufsize < 0 && errno != EINTR)
              longjmp(byebye,-1) ;
        }
        icurrchar = 0 ;
    }
    i_mode = INPUT_ACTIVE;
    switch(inbuf[icurrchar]) {
      case CTRL('L'):
        redoscr() ;
        icurrchar++ ;
        goto igetagain ;
      default:
        break ;
    }
    return inbuf[icurrchar++] ;
}

getdata(line,col,prompt,buf,len,echo,complete)
int line,col ;
char *prompt, *buf ;
int len, echo ;
int (*complete)() ;
{
    int ch ;
    int clen = 0 ;
    int x,y ;
    extern unsigned char scr_cols ;

    if(prompt) {
        move(line,col) ;
        prints("%s",prompt) ;
    }
    if(dumb_term) {
        while((ch = igetch()) != '\r') {
            if(ch == '\n')
              break ;
            if(ch == '\177' || ch == CTRL('H')) {
                if(clen == 0) {
                    bell() ;
                    continue ;
                }
                clen-- ;
                if(echo) {
                    ochar(CTRL('H')) ;
                    ochar(' ') ;
                    ochar(CTRL('H')) ;
                }
                continue ;
            }
#ifdef BIT8
	    if(!isprint2(ch)) {
#else
            if(!isprint(ch)) {
#endif
                if(echo)
                  bell() ;
                continue ;
            }
            if(clen >= len-1) {
                if(echo)
                  bell() ;
                continue ;
            }
            buf[clen++] = ch ;
            if(echo)
              ochar(ch) ;
        }
        buf[clen] = '\0' ;
        prints("\n") ;
        oflush() ;
        return clen ;
    }
    clrtoeol() ;
    getyx(&y,&x) ;
    while((ch = igetch()) != '\r') {
        if(ch == '\n')
          break ;
        if(ch == '\177' || ch == CTRL('H')) {
            if(clen == 0) {
                bell() ;
                continue ;
            }
            clen-- ;
            if(echo) {
                move(y,x+clen) ;
                prints(" ") ;
                move(y,x+clen) ;
            }
            continue ;
        }
#ifdef BIT8
	if(!isprint2(ch)) {
#else
        if(!isprint(ch)) {
#endif
            bell() ;
            continue ;
        }
        if(x+clen >= scr_cols || clen >= len-1) {
            bell() ;
            continue ;
        }
        buf[clen++] = ch ;
        if(echo)
          prints("%c",ch) ;
    }
    buf[clen] = '\0' ;
    if(echo)
      prints("\n") ;
    refresh() ;
    return clen ;
}

